/**
@license
Copyright (c) 2023 trading_peter
*/

import { LitElement, html, css } from 'lit';

class TpVSplitter extends LitElement {
  static get styles() {
    return [
      css`
        :host {
          display: grid;
          grid-template-columns: 1fr var(--tp-splitter-width, 5px) 1fr;
        }

        :host([lefthidden]),
        :host([righthidden]) {
          grid-template-rows: 1fr !important;
        }

        [hidden] {
          display: none !important;
        }

        #left,
        #right {
          position: relative;
          overflow: hidden;
        }

        #splitter {
          background: var(--tp-splitter-line-color, #3b3b3b);
          cursor: col-resize;
          opacity: 1;
        }

        #splitter:hover {
          background: var(--tp-splitter-line-color-hover, #007dd1);
          opacity: 0.5;
        }
      `
    ];
  }

  render() {
    const { leftHidden, rightHidden } = this;

    return html`
      <div id="left" part="left" ?hidden=${leftHidden}><slot name="left"></slot></div>
      <div id="splitter" part="splitter" @mousedown=${this._enableDrag}></div>
      <div id="right" part="right" ?hidden=${rightHidden}><slot name="right"></slot></div>
    `;
  }

  static get properties() {
    return {
      leftHidden: { type: Boolean },
      rightHidden: { type: Boolean },
      dragging: { type: Boolean },
      minSideWidth: { type: Number },
    };
  }

  constructor() {
    super();

    this._disableDrag = this._disableDrag.bind(this);
    this._resize = this._resize.bind(this);
  }

  get left() {
    return this.shadowRoot.querySelector('#left');
  }

  get right() {
    return this.shadowRoot.querySelector('#right');
  }

  disconnectedCallback() {
    super.disconnectedCallback();
    document.removeEventListener('mouseup', this._disableDrag)
    document.removeEventListener('mousemove', this._resize);
    document.body.style['userSelect'] = '';
  }

  _enableDrag(e) {
    document.addEventListener('mouseup', this._disableDrag)
    document.addEventListener('mousemove', this._resize);
    document.body.style['userSelect'] = 'none';
    this._startX = e.clientX;
    this._leftWidth = this.left.offsetWidth;
    this.dragging = true;
  }

  _disableDrag() {
    document.removeEventListener('mouseup', this._disableDrag)
    document.removeEventListener('mousemove', this._resize);
    document.body.style['userSelect'] = '';
    this.dragging = false;
    this.dispatchEvent(new CustomEvent('resize-done', { detail: this.style.gridTemplateColumns, bubbles: true, composed: true }));
  }

  _resize(e) {
    const delta = e.clientX - this._startX;
    const totalWidth = this.offsetWidth;
    const minWidth = this.minSideWidth || 0; // Minimum width in pixels for each side

    // Get splitter width from CSS variable or default to 5 pixels.
    const splitterWidth = parseInt(getComputedStyle(this).getPropertyValue('--tp-splitter-width')) || 5;
    
    // Calculate new left width while ensuring it stays within bounds.
    let newLeftWidth = Math.max(minWidth, Math.min(this._leftWidth + delta, totalWidth - splitterWidth - minWidth));
    
    this.style.gridTemplateColumns = `${newLeftWidth}px ${splitterWidth}px 1fr`;
    this.dispatchEvent(new CustomEvent('resized', { detail: null, bubbles: true, composed: true }));
  }
}

window.customElements.define('tp-vsplitter', TpVSplitter);
